﻿
(function () {
    var host = window.location.host;


    var WebBridge = new function() {
        var _this=this,
            _listeners={},
            _callbacks={},
            _messageId=0,
            _webViewListeners = {};
        
        function _on() {
            var i=0,target, type, cb;
            
            // if type is actually a target shift the params
            if (typeof arguments[0] != 'string') {
                target = arguments[i++];
            }

            /* Following code is used only to hook up to MSWebViewScriptNotify */
            if (target) {
                // this should not attach multiple times when being called multiple times.
                target.addEventListener("MSWebViewScriptNotify", _handleMSWebViewScriptNotify);
                _webViewListeners[target] = _webViewListeners[target] || 0;
                _webViewListeners[target]++;
            }

            type = arguments[i++];
            cb=arguments[i++];
            
            if (!_listeners[type]) {
                _listeners[type] = [];
            }
            _listeners[type].push({target:target,cb:cb});
        }
        
        function _off(type, cb) {
            var listeners, len, target;
            if (cb) {
                listeners = _listeners[type] || [];
                len = listeners.length;
                while(len--) {
                    if (listeners[len].cb === cb) {
                        _listeners[type].splice(len, 1);

                        /* The following code is used only for the unhooking of MSWebViewScriptNotify */
                        target = _listeners[type].target;
                        if (target && _webViewListeners[target]) {
                            _webViewListeners[target]--;
                            if (_webViewListeners[target].length <= 0) {
                                target.removeEventListener("MSWebViewScriptNotify", _handleMSWebViewScriptNotify);
                                delete _webViewListeners[target];
                            }
                        }
                    }
                }
            } else {
                _listeners[type] = [];
            }
        }
        
        function _broadcast() {
            var i=0,target, newTarget, type, payload, complete, asyncOp;
            
            // if type is actually a target shift the params
            if (typeof arguments[0] != 'string') {
                target = arguments[i++];
            }
            
            type = arguments[i++];
            payload = arguments[i++];
            complete = arguments[i++];

            // no payload ... only complete function
            if (payload instanceof Function) {
                complete = payload;
                payload = null;
            }

            // default payloads and complete functions
            payload = payload || {};
            try {
                if (typeof payload == 'string') {
                    payload = JSON.parse(payload);
                }
            } catch(e) {}
            complete = complete || function () {};

            _callbacks[_messageId]=complete;
      
            var message = {
                id: _messageId,
                type: type,
                host: host,
                payload: payload
            };
            
            newTarget = target ? (target.contentWindow || target) : (window.opener || window.parent || window);
            // do not check for notify like this: window.external.notify.  Will not return true in webview land
            if (target && window.external && 'notify' in window.external) {
                window.external.notify(JSON.stringify(message));
            // target could be a iframe or a window opened by window.opener
            } else if (newTarget.postMessage) {
                newTarget.postMessage(message, "*");
            } else if (target.invokeScriptAsync) {
                asyncOp = target.invokeScriptAsync("receiveInvokeScriptAsync", [JSON.stringify(message)]);
                asyncOp.oncomplete = function () { };
                asyncOp.onerror = function () { };
                asyncOp.start();
            }
            _messageId += 1;
        }
        
        function _handleMSWebViewScriptNotify(e) {
            _handleMessage({
                data: JSON.parse(e.value)
            });
        }

        function _handleMessage(e) {
            // e.detail is from the event coming out of receiveInvokeScriptAsync
            var message = e.data || e.detail, listeners, response, source = e.source,cb;
           
            if (message.type === '_complete') {
               cb=_callbacks[message.id];
               cb && (cb)(message.response);
               delete _callbacks[message.id];
                
            } else {
                listeners=_listeners[message.type];
                if (listeners) {
                    listeners.forEach(function(listener) {
                        if (listener.cb && (!listener.target || (listener.target.contentWindow==source))) {
                            response=((listener.cb)(message.payload));
                        }
                        
                    });
                }

                if (source) {
                    source.postMessage({type:"_complete",id:message.id, response:response}, "*");
                }
            }
        };

        function _init() {

            window.addEventListener("message", _handleMessage);

            /* Used for Windows webview
                http://msdn.microsoft.com/en-us/library/windows/apps/dn301841.aspx
            */
            window.receiveInvokeScriptAsync = function(message) {
                _handleMessage({
                    data: JSON.parse(message)
                });
            };
        }
        

        
        _init();
        
        return {
            // events 1:many
            on:_on,
            off:_off,
            broadcast:_broadcast,
            
            // attributes 1 : 1
            set:_broadcast,
            get:_broadcast,
            onGet:_on,
            onSet:_on
        }
    }
        
    window.WebBridge = WebBridge;
})();